#include "MD2Loader.h"

int loadMD2(MD2Model*md2, char*filename) {
     int i;
	 FILE*model = NULL;
	 model = fopen(filename, "rb");//on fais la lecture en mode byte
	 if(model > 0) {
	     fread(&md2->header, sizeof(Header), 1, model);
		 if(md2->header.ident != 844121161 || md2->header.version != 8) {//si c'est bien le bon type de fichier md2 (844121161 -->> valeurs ascii de "IDPA")
		     printf("Error: Invalid md2 file");
			 return 0;
		 }
		 else {
		     md2->texture = (Textures*)malloc(sizeof(Textures)*md2->header.numbertexture);//on alloue dynamiquement les tableaux utilise par le modle
			 md2->coordtexture = (CoordsTextures*)malloc(sizeof(CoordsTextures)*md2->header.numbercoordtexture);
			 md2->triangle = (Triangles*)malloc(sizeof(Triangles)*md2->header.numbertriangle);
			 md2->frame = (Frames*)malloc(sizeof(Frames)*md2->header.numberframe);
			 
			 //on se dplace dans le fichier suivant les valeurs du header que l'on a prcdemment lu (c'est  dire les offset) pour stocker les valeurs des coordonnes de texture...
			 fseek(model, md2->header.offsettexture, SEEK_SET);
			 fread(md2->texture, sizeof(Textures), md2->header.numbertexture, model);
			 fseek(model, md2->header.offsetcoordtexture, SEEK_SET);
			 fread(md2->coordtexture, sizeof(CoordsTextures), md2->header.numbercoordtexture, model);
			 fseek(model, md2->header.offsettriangle, SEEK_SET);
			 fread(md2->triangle, sizeof(Triangles), md2->header.numbertriangle, model);
			 
			 fseek(model, md2->header.offsetframe, SEEK_SET);
			 
			 for(i=0; i<md2->header.numberframe; i++) {
			     md2->frame[i].framevertex = (FramesVertex*)malloc(sizeof(FramesVertex)*md2->header.numbervertex);
				 
				 fread(md2->frame[i].scale, sizeof(md2->frame[i].scale), 1, model);
				 fread(md2->frame[i].translate, sizeof(md2->frame[i].translate), 1, model);
				 fread(md2->frame[i].name, sizeof(md2->frame[i].name), 1, model);
				 fread(md2->frame[i].framevertex, sizeof(FramesVertex), md2->header.numbervertex, model);
			 }
			 fseek(model, md2->header.offsetend, SEEK_SET);//on place le curseur de lecture  la fin du fichier
		 }
	 }
	 fclose(model);
	 return 1;
}
	 
void MD2Display(MD2Model*md2, int currentframe, int texture) {
     int i;
	 int j;
	 if(texture != 0) glBindTexture(GL_TEXTURE_2D, texture);
	 glBegin(GL_TRIANGLES);
	 for(i=0; i<md2->header.numbertriangle; i++) {//pour chaque paquet de triangle (contenant 3 coordonnes de texture et de vertices)
	     for(j=0; j<3; j++) {//dont 3 coordonnes de chaque type
		     if(texture != 0) glTexCoord2f((float)md2->coordtexture[md2->triangle[i].xy[j]].x/md2->header.texturewidth, (float)md2->coordtexture[md2->triangle[i].xy[j]].y/md2->header.textureheight);//on applique les coordonnes de texture au md2 charg en dcompressant ses coordonnes de texture en divisant celles-ci par la taille en largeur et hauteur de la texture pour situer celles-ci sur un intervalle compris entre 0.0 et 1.0
			 glVertex3f((md2->frame[currentframe].scale[0]*md2->frame[currentframe].framevertex[md2->triangle[i].vertex[j]].vertex[0])+md2->frame[currentframe].translate[0], (md2->frame[currentframe].scale[1]*md2->frame[currentframe].framevertex[md2->triangle[i].vertex[j]].vertex[1])+md2->frame[currentframe].translate[1], (md2->frame[currentframe].scale[2]*md2->frame[currentframe].framevertex[md2->triangle[i].vertex[j]].vertex[2])+md2->frame[currentframe].translate[2]);
		 }
	 }
	 glEnd();
}

void MD2Animation(MD2Model*md2, float speed, float debut, float fin) {
	 md2->currentframe+=speed;
	 if(fin < 0) fin = md2->header.numberframe-1;
	 if(md2->currentframe > md2->header.numberframe-1 || md2->currentframe < 0.0f || md2->currentframe > fin || md2->currentframe < debut) {
	     md2->currentframe = debut;
	 }
}

void freeMD2(MD2Model*md2) {
	 int i;
	 free(md2->texture);
	 free(md2->coordtexture);
	 free(md2->frame);
	 free(md2->triangle);
	 for(i=0; md2->header.numbervertex; i++) {
	     free(md2->frame[i].framevertex);
	 }
}
