#define PSP

#ifdef PSP

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_framerate.h>
#include <stdio.h> 
#include <stdlib.h>   
#include <pspctrl.h>
#include <pspuser.h>
#include <time.h>
#include <math.h>
#include <pspaudio.h>
#include <pspaudiolib.h>

PSP_MODULE_INFO("editorlabKillHomer3D", 0, 1, 1); 
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

int exit_callback(int arg1, int arg2, void *common) {
     sceKernelExitGame();
     return 0;
}

int CallbackThread(SceSize args, void *argp) {
     int cbid;
     cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
     sceKernelRegisterExitCallback(cbid);
     sceKernelSleepThreadCB();
     return 0;
}

int SetupCallbacks(void) {
     int thid = 0;
     thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
     if(thid >= 0) {
         sceKernelStartThread(thid, 0, 0);
     }
     return thid;
}

#define exit(X) sceKernelExitGame()

#endif

#ifdef WINDOWS

#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_framerate.h>

#endif 

#ifdef WII

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <SDL/sdl.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_framerate.h>
#include <gccore.h>
#include <wiiuse/wpad.h>

#endif

#define width 480
#define height 272
#define nbmaxwidth (480/14)+1//+1 car c'est pour remplir la ligne manquante de la fnetre
#define nbmaxheight (272/14)+1

typedef struct Case {
	 SDL_Surface*img;
	 int x;
	 int y;
	 int xsprite;
	 int ysprite;
	 int wsprite;
	 int hsprite;
	 int statut;
	 int infotype;
	 char type;
}
Case;
Case editcase[nbmaxheight][nbmaxwidth];

//char alphabet[] = " !#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[/]^_`abcdefghijklmnopqrstuvwxyz{|}~";//because /\:?"<>| are not supported :p
char alphabet[] = " !#$%&'()$+,-.$0123456789$;$=$$@ABCDEFGHIJKLMNOPQRSTUVWXYZ[$]^_`abcdefghijklmnopqrstuvwxyz{$}~";
char filename[256];
int curseur = 0;
int currentletter = 0;
int currentlettername = 0;

void drawKeyboard(SDL_Surface*ecran, SDL_Surface*keyboard, TTF_Font*normal, SDL_Rect cursorpos) {
		 SDL_Surface*cursor;
		 SDL_Surface*font;
		 SDL_Color color = {0, 0, 0};
		 SDL_Color colorcursor = {255, 0, 0};
		 SDL_Rect fontpos = {45, 210};
		 SDL_Rect keyboardpos = {50, 100};
		 font = TTF_RenderText_Solid(normal, filename, color);
		 cursor = TTF_RenderText_Solid(normal, "_", colorcursor);
		 SDL_BlitSurface(font, NULL, ecran, &fontpos);
		 SDL_FreeSurface(font);
		 SDL_BlitSurface(keyboard, NULL, ecran, &keyboardpos);
		 SDL_BlitSurface(cursor, NULL, ecran, &cursorpos);
		 SDL_FreeSurface(cursor);
}

void blitCase(SDL_Surface*ecran, Case*caseedit, SDL_Surface*casesurface) {
     SDL_Rect casespriteposition = {caseedit->xsprite, caseedit->ysprite, caseedit->wsprite, caseedit->hsprite};
	 SDL_Rect caseposition = {caseedit->x, caseedit->y, 0, 0};
	 switch(caseedit->infotype) {
	     case 0:
		     caseedit->type = '1';
		 break;
		 case 1:
		     caseedit->type = '0';
	     break;
		 case 2:
		     caseedit->type = 'P';
		 break;
		 case 3:
		     caseedit->type = '+';
		 break;
		 case 4:
		     caseedit->type = 'S';
		 break;
	 }
	 if(caseedit->statut == 1) {
	     SDL_BlitSurface(casesurface, &casespriteposition, ecran, &caseposition);
	 }
}

#ifdef PSP
int main(int argc, char *argv[]) {
	 if(SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO) == -1) {
	     fprintf(stderr, "Probleme de parametrage de la fenetre");
		 exit(EXIT_FAILURE);
	 }
	 TTF_Init();
	 FPSmanager fpsmanager;
	 SDL_initFramerate(&fpsmanager);
	 SDL_setFramerate(&fpsmanager, 60);
	 
	 TTF_Font*normal;
	 SDL_Surface*ecran = NULL;
	 SDL_Surface*font = NULL;
	 SDL_Surface*fontwarning = NULL;
	 SDL_Surface*editsurface = NULL;
	 SDL_Surface*cursor = NULL;	 
	 SDL_Surface*menu = NULL;
	 SDL_Surface*cursormenu = NULL;
	 SDL_Surface*clavier = NULL;
	 SDL_Surface*writing = NULL;
	 
	 ecran = SDL_SetVideoMode(width, height, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	 
	 normal = TTF_OpenFont("./data/calibri.ttf", 18);
	 menu = IMG_Load("./data/menu.png");
	 cursormenu = IMG_Load("./data/cursormenu.png");
	 editsurface = IMG_Load("./data/editsurface.png");
	 cursor = IMG_Load("./data/curseur.png");
	 clavier = IMG_Load("./data/keyboard.png");
	 SDL_Rect editsurfacepos = {0, 0, 70, 14};
	 SDL_Rect editcubepos = {0, 0, 14, 14};
	 SDL_Rect fontpos = {300, 20};
	 SDL_Rect fontwarningpos = {0, 150};
	 SDL_Rect menupos = {20, 20};
	 SDL_Rect cursormenupos = {30, 34};
	 SDL_Color color = {255,50,50};
	 SDL_Color warningcolor = {255, 0, 0};
	 SDL_Rect curseurnamepos = {50, 97};
	 SDL_Rect writingpos = {47, 195};
	 
	 char infoelement[64];
	 char filesaving[64];
	 int checkwarning = 0;
	 int checkplayer = 0;//variable vrifiant si tous les lmenty ont t mis sur la map avant enregistrement
	 int checkmonster = 0;
	 int checkexit = 0;
	 int i;
	 int j;
	 int elementtype = 0;//statut mur
	 int statewidth = 0;
	 int stateheight = 0;
	 int menustate = 0;
	 int menuselection = 0;
	 int delay = 0;
	 
	 int keyuprepeat = 0;
	 int keydownrepeat = 0;
	 int keyrightrepeat = 0;
	 int keyleftrepeat = 0;
	 int keycrossrepeat = 0;
	 int oldcrosscountfirst = 0;
	 int oldcrosscountend = 0;	 
	 
	 for(i=0; i<nbmaxheight; i++) {
	     for(j=0; j<nbmaxwidth; j++) {
			 editcase[i][j].x = j*14;
		     editcase[i][j].y = i*14;
			 editcase[i][j].xsprite = 14;
			 editcase[i][j].ysprite = 0;
			 editcase[i][j].wsprite = 70;
			 editcase[i][j].hsprite = 14;
			 editcase[i][j].infotype = 1;
		     editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		 }
	 }
	 
	 FILE*save = NULL;
	 
	 writing = TTF_RenderText_Solid(normal, "Write, using this keyboard, the name of your map:", color);
	 fontwarning = TTF_RenderText_Solid(normal, "Saving failed: please, verify if you have all elements' type in your map", warningcolor);
	 
	 mkdir(".../data/custommap");
	 
	 sceCtrlSetSamplingCycle(0);
     sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	 SceCtrlData pad;
	 SetupCallbacks();
	 int oldpad = 0;
	 
	 SceUID ebootload = pspSdkLoadStartModule("../loadeboot.prx", PSP_MEMORY_PARTITION_KERNEL);
	 if(ebootload < 0) {
		     printf("Attention: erreur 0x08X chargement", ebootload);
			 exit(0);
	 }
	 FILE*pathfile;
	 pathfile = fopen("../path.dat", "r");
	 char path[64];	
	 char pathsaved[64];
	 
	 while(1) {
	     sceCtrlPeekBufferPositive(&pad, 1);		 
		 
		 if(oldpad != pad.Buttons) {
		         if(pad.Buttons & PSP_CTRL_CROSS) {
					     if(menustate == 1) {
						     if(menuselection == 0) {
		                         for(i=0; i<nbmaxheight; i++) {
	                                 for(j=0; j<nbmaxwidth; j++) {
										 editcase[i][j].x = j*13;
		                                 editcase[i][j].y = i*13;
			                             editcase[i][j].xsprite = 13;
			                             editcase[i][j].ysprite = 0;
			                             editcase[i][j].wsprite = 52;
			                             editcase[i][j].hsprite = 13;
			                             editcase[i][j].infotype = 1;
		                                 editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		                             }
	                             }
								 for(i=0; i < currentlettername-1; i++) {
								     filename[i] = '\0';
                				 }
								 currentletter = 0;
								 currentlettername = 0;
								 curseurnamepos.x = 50;
								 curseurnamepos.y = 97;
								 editsurfacepos.x = 0;
								 editsurfacepos.y = 0;
								 stateheight = 0;
								 statewidth = 0;
								 checkplayer = 0;
								 checkmonster = 0;
								 checkexit = 0;
								 oldcrosscountfirst = SDL_GetTicks();
	                             menustate = 0;
                             }	 
							 else if(menuselection == 1) {//selection sauvegarde
							     menustate = 3;
						     }
							 else if(menuselection == 2) {
							     if(pathfile > 0) {
								     while(fgets(path, sizeof(path), pathfile) > 0) {
									     if(path[0] == 'g') {
										     sscanf(path+2, "%s", &pathsaved);
										 }
									 }
									 fclose(pathfile);
								 }
								 else if(pathfile <= 0) {
								     sprintf(pathsaved, "ms0:/PSP/GAME/labKillHomer3D/EBOOT.PBP");
									 fclose(pathfile);
								 }
								 if(loadEboot(pathsaved) == 1) {
	                                 SDL_FreeSurface(ecran);
									 SDL_FreeSurface(cursor);
	                                 SDL_FreeSurface(font);
	                                 SDL_FreeSurface(menu);
	                                 SDL_FreeSurface(cursormenu);
	                                 SDL_FreeSurface(fontwarning);
	                                 SDL_FreeSurface(editsurface);
	                                 SDL_FreeSurface(clavier);
	                                 TTF_CloseFont(normal);
	                                 TTF_Quit();
	                                 SDL_Quit();
								 }
								 //exit(0);
							 }
						 }
						 else if(menustate == 3) {
							 if(curseurnamepos.y == 97) { 
							     if(curseurnamepos.x > 50) {
								     if(currentlettername < 30) {
									     filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 50) {
								     if(currentlettername > 0) {   
   										 filename[currentlettername-1] = '\0';
									     currentlettername--;
									 }
								 }
							 }
							 else if(curseurnamepos.y > 97 && curseurnamepos.y < 177) {
							     if(currentlettername < 30) {
								     filename[currentlettername] = alphabet[currentletter];
								     currentlettername++;
								 }
							 }
							 else if(curseurnamepos.y == 177) {
							     if(curseurnamepos.x < 290) {
								     if(currentlettername < 30) {    
										 filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 290) {
						             if(checkplayer > 0 && checkmonster > 0 && checkexit > 0) {								 
									 sprintf(filesaving, ".../data/custommap/%s.map", filename);
									 save = fopen(filesaving, "w+");
								     if(save > 0) {
										 for(i=0; i<nbmaxheight; i++) {
						                     for(j=0; j<nbmaxwidth; j++) {
										     
												 if(j == nbmaxwidth-1) {
										             fprintf(save, "%c\n", editcase[i][j].type);
									             }
									             if(j != nbmaxwidth-1) {
								                     fprintf(save, "%c", editcase[i][j].type);
									             }
						                     }
						                 }
									 }
						             fclose(save);
								     oldcrosscountfirst = SDL_GetTicks();
	                                 menustate = 0;
							         }
								     else if(checkplayer <= 0 || checkexit <= 0 || checkmonster <= 0) {
									     delay = 0;
									     checkwarning = 1;
								         oldcrosscountfirst = SDL_GetTicks();
	                                     menustate = 0;
								     }	
								 }
							 }
						 }
	             }			         
				 else if(pad.Buttons & PSP_CTRL_START) {
					     if(menustate == 0) menustate = 1;
						 else if(menustate == 1) {
						     checkplayer = 0;
							 checkmonster = 0;
							 checkexit = 0;
							 menustate = 0;
						 }
						 if(menustate == 1) {//avant d'entrer dans le menu on check si tous les paramtres (perso, monstre et sortir sont saisie) :D
								 for(i=0; i<nbmaxheight; i++) {
						             for(j=0; j<nbmaxwidth; j++) {
										 if(editcase[i][j].type == 'P') checkplayer++;
				                         if(editcase[i][j].type == '+') checkmonster++;
										 if(editcase[i][j].type == 'S') checkexit++;
						             }
						         }
						 }
                 }
		         else if(pad.Buttons & PSP_CTRL_RTRIGGER) {
					     if(menustate == 0) {
						     if(editcubepos.x < 56) {
						         editcubepos.x+=14;
							     elementtype++;
						     }
						     else if(editcubepos.x >= 56) {
						         editcubepos.x = 0;
							     elementtype = 0;
						     }
						 }
		         }				 
		 }
		 oldpad = pad.Buttons;
		 
		 if((pad.Buttons & PSP_CTRL_RIGHT && keyrightrepeat == 0) || (pad.Lx > 200)) {
					     if(menustate == 0) {
						     if(editsurfacepos.x < width-14) {
						         editsurfacepos.x+=14;
							     statewidth++;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x < 290) {
						         curseurnamepos.x+=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter++;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 66) {
									     currentletter++;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 290) {
									     currentletter++;
									 }
								 }
						     }
						 }
						 keyrightrepeat = 5;
         }
		 else keyrightrepeat--;
		 if(keyrightrepeat < 0) keyrightrepeat = 5;
		 
		 if((pad.Buttons & PSP_CTRL_LEFT && keyleftrepeat == 0) || (pad.Lx < 30)) {
					     if(menustate == 0) {
						     if(editsurfacepos.x > 0) {
						         editsurfacepos.x-=14;
							     statewidth--;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x > 50) {
						         curseurnamepos.x-=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter--;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 50) {
									     currentletter--;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 274) {
									     currentletter--;
									 }
								 }
						     }
						 }
						 keyleftrepeat = 5;
		 }
		 else keyleftrepeat--;
		 if(keyleftrepeat < 0) keyleftrepeat = 5;
		 
		 if((pad.Buttons & PSP_CTRL_UP && keyuprepeat == 0) || (pad.Ly < 30)) {
					     if(menustate == 0) {
						     if(editsurfacepos.y > 0) {
						         editsurfacepos.y-=14;
							     stateheight--;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y > 34) {
							     cursormenupos.y-=14;
								 menuselection--;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y > 97) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter-=16;
						         }
							     else if(curseurnamepos.x == 50) {
							         if(curseurnamepos.y == 113) {
								         currentletter-=15;
								     }
								     else currentletter-=16;
							     }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 177) {
									     currentletter-=15;
									 }
									 else currentletter-=16;
								 }
								 curseurnamepos.y-=16;
							 }
						 }
						 keyuprepeat = 5;
		 }
		 else keyuprepeat--;
		 if(keyuprepeat < 0) keyuprepeat = 5;
		 
		 if((pad.Buttons & PSP_CTRL_DOWN && keydownrepeat == 0) || (pad.Ly > 200)) {
					     if(menustate == 0) {
						     if(editsurfacepos.y < height-14) {
						         editsurfacepos.y+=14;
							     stateheight++;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y < 62) {
							     cursormenupos.y+=14;
								 menuselection++;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y < 176) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter+=16;
								 }
								 else if(curseurnamepos.x == 50) {
								     if(curseurnamepos.y == 97) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 161) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
							     curseurnamepos.y+=16;
						     }
						 }
						 keydownrepeat = 5;
		 }	
		 else keydownrepeat--;
		 if(keydownrepeat < 0) keydownrepeat = 5;
		 
		 if(pad.Buttons & PSP_CTRL_CROSS && keycrossrepeat == 0) {
			 oldcrosscountend = SDL_GetTicks()-oldcrosscountfirst;
			 if(oldcrosscountend > 250) {
				 if(menustate == 0) {
				     editcase[stateheight][statewidth].statut = 1;
				     editcase[stateheight][statewidth].x = editsurfacepos.x;
				     editcase[stateheight][statewidth].y = editsurfacepos.y;
				     editcase[stateheight][statewidth].xsprite = editcubepos.x;
				     editcase[stateheight][statewidth].ysprite = editcubepos.y;
				     editcase[stateheight][statewidth].wsprite = editcubepos.w;
				     editcase[stateheight][statewidth].hsprite = editcubepos.h;
				     editcase[stateheight][statewidth].infotype = elementtype;
			     }
			     keycrossrepeat = 5;
			 }
	     }
		 else keycrossrepeat--;
		 if(keycrossrepeat < 0) keycrossrepeat = 5;		 
		 
		 SDL_FillRect(ecran, NULL, SDL_MapRGB(ecran->format, 255, 255, 255));
		 
		 if(elementtype == 0) sprintf(infoelement, "Current selection: Wall");
		 else if(elementtype == 1) sprintf(infoelement, "Current selection: Empty");
		 else if(elementtype == 2) sprintf(infoelement, "Current selection: Player");
		 else if(elementtype == 3) sprintf(infoelement, "Current selection: Homer");
		 else if(elementtype == 4) sprintf(infoelement, "Current selection: Exit");
		 font = TTF_RenderText_Solid(normal, infoelement, color);
		 
		 for(i=0; i<nbmaxheight; i++) {
	         for(j=0; j<nbmaxwidth; j++) {
				 blitCase(ecran, &editcase[i][j], editsurface);
		     }
	     }
		 
		 
		 if(menustate == 0) {
		     if(delay >= 100) {
			     checkwarning = 0;
			 }
		 }
		 else if(menustate == 3)  {
		     SDL_BlitSurface(writing, NULL, ecran, &writingpos);
			 drawKeyboard(ecran, clavier, normal, curseurnamepos);
		 }
		 
		 SDL_BlitSurface(cursor, NULL, ecran, &editsurfacepos);
		 SDL_BlitSurface(font, NULL, ecran, &fontpos);
		 SDL_FreeSurface(font);
		 if(menustate == 1) {
		     SDL_BlitSurface(menu, NULL, ecran, &menupos);
		     SDL_BlitSurface(cursormenu, NULL, ecran, &cursormenupos);
		 }
		 
		 delay++;
		 if(delay < 100 && checkwarning == 1) SDL_BlitSurface(fontwarning, NULL, ecran, &fontwarningpos);
		 
		 SDL_framerateDelay(&fpsmanager);
		 SDL_Flip(ecran);
	 }
	 fclose(pathfile);
	 SDL_FreeSurface(ecran);
	 SDL_FreeSurface(cursor);
	 SDL_FreeSurface(font);
	 SDL_FreeSurface(menu);
	 SDL_FreeSurface(cursormenu);
	 SDL_FreeSurface(fontwarning);
	 SDL_FreeSurface(editsurface);
	 SDL_FreeSurface(clavier);
	 TTF_CloseFont(normal);
	 TTF_Quit();
	 SDL_Quit();
	 return 0;
}
#endif

#ifdef WINDOWS
int main(int argc, char *argv[]) {
	 if(SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO) == -1) {
	     fprintf(stderr, "Probleme de parametrage de la fenetre");
		 exit(EXIT_FAILURE);
	 }
	 TTF_Init();
	 FPSmanager fpsmanager;
	 SDL_initFramerate(&fpsmanager);
	 SDL_setFramerate(&fpsmanager, 60);
	 
	 SDL_Event event;
	 TTF_Font*normal;
	 SDL_Surface*ecran = NULL;
	 SDL_Surface*font = NULL;
	 SDL_Surface*fontwarning = NULL;
	 SDL_Surface*editsurface = NULL;
	 SDL_Surface*cursor = NULL;	 
	 SDL_Surface*menu = NULL;
	 SDL_Surface*cursormenu = NULL;
	 SDL_Surface*clavier = NULL;
	 SDL_Surface*writing = NULL;
	 
	 ecran = SDL_SetVideoMode(width, height, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	 SDL_WM_SetCaption("Editor of map labKillHomer3D by pspflashsystem", NULL);
	 
	 normal = TTF_OpenFont("./data/calibri.ttf", 18);
	 menu = IMG_Load("./data/menu.png");
	 cursormenu = IMG_Load("./data/cursormenu.png");
	 editsurface = IMG_Load("./data/editsurface.png");
	 cursor = IMG_Load("./data/curseur.png");
	 clavier = IMG_Load("./data/keyboard.png");
	 SDL_Rect editsurfacepos = {0, 0, 70, 14};
	 SDL_Rect editcubepos = {0, 0, 14, 14};
	 SDL_Rect fontpos = {300, 20};
	 SDL_Rect fontwarningpos = {0, 150};
	 SDL_Rect menupos = {20, 20};
	 SDL_Rect cursormenupos = {30, 34};
	 SDL_Color color = {255,50,50};
	 SDL_Color warningcolor = {255, 0, 0};
	 SDL_Rect curseurnamepos = {50, 97};
	 SDL_Rect writingpos = {47, 195};
	 
	 char infoelement[64];
	 char filesaving[64];
	 int checkwarning = 0;
	 int checkplayer = 0;//variable vrifiant si tous les lmenty ont t mis sur la map avant enregistrement
	 int checkmonster = 0;
	 int checkexit = 0;
	 int i;
	 int j;
	 int elementtype = 0;//statut mur
	 int statewidth = 0;
	 int stateheight = 0;
	 int menustate = 0;
	 int menuselection = 0;
	 int delay = 0;
	 
	 for(i=0; i<nbmaxheight; i++) {
	     for(j=0; j<nbmaxwidth; j++) {
			 editcase[i][j].x = j*14;
		     editcase[i][j].y = i*14;
			 editcase[i][j].xsprite = 14;
			 editcase[i][j].ysprite = 0;
			 editcase[i][j].wsprite = 70;
			 editcase[i][j].hsprite = 14;
			 editcase[i][j].infotype = 1;
		     editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		 }
	 }
	 
	 FILE*save = NULL;
	 
	 writing = TTF_RenderText_Solid(normal, "Write, using this keyboard, the name of your map:", color);
	 fontwarning = TTF_RenderText_Solid(normal, "Saving failed: please, verify if you have all elements' type in your map", warningcolor);
	 
	 mkdir("./data/custommap");
	 
	 SDL_EnableKeyRepeat(5, 150);
	 
	 while(1) {
		 
		 while(SDL_PollEvent(&event)) {
		 switch(event.type) {
		     case SDL_QUIT:
			     exit(0);
			 break;
			 case SDL_KEYDOWN: 
			     switch(event.key.keysym.sym) {
				     case SDLK_r:
					     if(menustate == 0) {
						     if(editcubepos.x < 56) {
						         editcubepos.x+=14;
							     elementtype++;
						     }
						     else if(editcubepos.x >= 56) {
						         editcubepos.x = 0;
							     elementtype = 0;
						     }
						 }
					 break;
					 case SDLK_RIGHT:
					     if(menustate == 0) {
						     if(editsurfacepos.x < width-14) {
						         editsurfacepos.x+=14;
							     statewidth++;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x < 290) {
						         curseurnamepos.x+=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter++;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 66) {
									     currentletter++;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 290) {
									     currentletter++;
									 }
								 }
						     }
						 }
					 break;
					 case SDLK_LEFT:
					     if(menustate == 0) {
						     if(editsurfacepos.x > 0) {
						         editsurfacepos.x-=14;
							     statewidth--;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x > 50) {
						         curseurnamepos.x-=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter--;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 50) {
									     currentletter--;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 274) {
									     currentletter--;
									 }
								 }
						     }
						 }
					 break;
					 case SDLK_UP:
					     if(menustate == 0) {
						     if(editsurfacepos.y > 0) {
						         editsurfacepos.y-=14;
							     stateheight--;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y > 34) {
							     cursormenupos.y-=14;
								 menuselection--;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y > 97) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter-=16;
						         }
							     else if(curseurnamepos.x == 50) {
							         if(curseurnamepos.y == 113) {
								         currentletter-=15;
								     }
								     else currentletter-=16;
							     }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 177) {
									     currentletter-=15;
									 }
									 else currentletter-=16;
								 }
								 curseurnamepos.y-=16;
							 }
						 }
					 break;
					 case SDLK_DOWN:
					     if(menustate == 0) {
						     if(editsurfacepos.y < height-14) {
						         editsurfacepos.y+=14;
							     stateheight++;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y < 62) {
							     cursormenupos.y+=14;
								 menuselection++;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y < 176) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter+=16;
								 }
								 else if(curseurnamepos.x == 50) {
								     if(curseurnamepos.y == 97) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 161) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
							     curseurnamepos.y+=16;
						     }
						 }
					 break;
					 case SDLK_x:
					     if(menustate == 0) {
							 editcase[stateheight][statewidth].statut = 1;
						     editcase[stateheight][statewidth].x = editsurfacepos.x;
						     editcase[stateheight][statewidth].y = editsurfacepos.y;
						     editcase[stateheight][statewidth].xsprite = editcubepos.x;
						     editcase[stateheight][statewidth].ysprite = editcubepos.y;
						     editcase[stateheight][statewidth].wsprite = editcubepos.w;
						     editcase[stateheight][statewidth].hsprite = editcubepos.h;
							 editcase[stateheight][statewidth].infotype = elementtype;
						 }
						 else if(menustate == 1) {
						     if(menuselection == 0) {
		                         for(i=0; i<nbmaxheight; i++) {
	                                 for(j=0; j<nbmaxwidth; j++) {
										 editcase[i][j].x = j*13;
		                                 editcase[i][j].y = i*13;
			                             editcase[i][j].xsprite = 13;
			                             editcase[i][j].ysprite = 0;
			                             editcase[i][j].wsprite = 52;
			                             editcase[i][j].hsprite = 13;
			                             editcase[i][j].infotype = 1;
		                                 editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		                             }
	                             }
								 for(i=0; i < currentlettername-1; i++) {
								     filename[i] = '\0';
				
                				 }
								 currentletter = 0;
								 currentlettername = 0;
								 curseurnamepos.x = 50;
								 curseurnamepos.y = 97;
								 editsurfacepos.x = 0;
								 editsurfacepos.y = 0;
								 stateheight = 0;
								 statewidth = 0;
								 checkplayer = 0;
								 checkmonster = 0;
								 checkexit = 0;
	                             menustate = 0;
                             }	 
							 else if(menuselection == 1) {//selection sauvegarde
							     menustate = 3;
						     }
							 else if(menuselection == 2) {
							     exit(0);
							 }
						 }
						 else if(menustate == 3) {
							 if(curseurnamepos.y == 97) { 
							     if(curseurnamepos.x > 50) {
								     if(currentlettername < 30) {
									     filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 50) {
								     if(currentlettername > 0) {   
   										 filename[currentlettername-1] = '\0';
									     currentlettername--;
									 }
								 }
							 }
							 else if(curseurnamepos.y > 97 && curseurnamepos.y < 177) {
							     if(currentlettername < 30) {
								     filename[currentlettername] = alphabet[currentletter];
								     currentlettername++;
								 }
							 }
							 else if(curseurnamepos.y == 177) {
							     if(curseurnamepos.x < 290) {
								     if(currentlettername < 30) {    
										 filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 290) {
						             if(checkplayer > 0 && checkmonster > 0 && checkexit > 0) {								 
									 sprintf(filesaving, "./data/custommap/%s.map", filename);
									 save = fopen(filesaving, "w+");
								     if(save > 0) {    
										 for(i=0; i<nbmaxheight; i++) {
						                     for(j=0; j<nbmaxwidth; j++) {
												 if(j == nbmaxwidth-1) {
										             fprintf(save, "%c\n", editcase[i][j].type);
									             }
									             if(j != nbmaxwidth-1) {
								                     fprintf(save, "%c", editcase[i][j].type);
									             }
						                     }
						                 }
									 }
						             fclose(save);
								     menustate = 0;
							         }
								     else if(checkplayer <= 0 || checkexit <= 0 || checkmonster <= 0) {
									 delay = 0;
									 checkwarning = 1;
								     menustate = 0;
								     }	
								 }
							 }
						 }
					 break;
					 case SDLK_SPACE:
					     if(menustate == 0) menustate = 1;
						 else if(menustate == 1) {
						     checkplayer = 0;
							 checkmonster = 0;
							 checkexit = 0;
							 menustate = 0;
						 }
						 if(menustate == 1) {//avant d'entrer dans le menu on check si tous les paramtres (perso, monstre et sortir sont saisie) :D
								 for(i=0; i<nbmaxheight; i++) {
						             for(j=0; j<nbmaxwidth; j++) {
										 if(editcase[i][j].type == 'P') checkplayer++;
				                         if(editcase[i][j].type == '+') checkmonster++;
										 if(editcase[i][j].type == 'S') checkexit++;
						             }
						         }
						 }
					 break;
					 case SDLK_ESCAPE:
					     exit(0);
					 break;
				 }
			 break;			 
		 }
		 }
		 
		 SDL_FillRect(ecran, NULL, SDL_MapRGB(ecran->format, 255, 255, 255));
		 
		 if(elementtype == 0) sprintf(infoelement, "Current selection: Wall");
		 else if(elementtype == 1) sprintf(infoelement, "Current selection: Empty");
		 else if(elementtype == 2) sprintf(infoelement, "Current selection: Player");
		 else if(elementtype == 3) sprintf(infoelement, "Current selection: Homer");
		 else if(elementtype == 4) sprintf(infoelement, "Current selection: Exit");
		 font = TTF_RenderText_Solid(normal, infoelement, color);
		 
		 for(i=0; i<nbmaxheight; i++) {
	         for(j=0; j<nbmaxwidth; j++) {
				 blitCase(ecran, &editcase[i][j], editsurface);
		     }
	     }
		 
		 
		 if(menustate == 0) {
		     if(delay >= 100) {
			     checkwarning = 0;
			 }
		 }
		 else if(menustate == 3)  {
		     SDL_BlitSurface(writing, NULL, ecran, &writingpos);
			 drawKeyboard(ecran, clavier, normal, curseurnamepos);
		 }
		 
		 SDL_BlitSurface(cursor, NULL, ecran, &editsurfacepos);
		 SDL_BlitSurface(font, NULL, ecran, &fontpos);
		 SDL_FreeSurface(font);
		 if(menustate == 1) {
		     SDL_BlitSurface(menu, NULL, ecran, &menupos);
		     SDL_BlitSurface(cursormenu, NULL, ecran, &cursormenupos);
		 }
		 
		 delay++;
		 if(delay < 100 && checkwarning == 1) SDL_BlitSurface(fontwarning, NULL, ecran, &fontwarningpos);
		 
		 SDL_framerateDelay(&fpsmanager);
		 SDL_Flip(ecran);
	 }
	 SDL_FreeSurface(ecran);	 
	 SDL_FreeSurface(cursor);
	 SDL_FreeSurface(font);
	 SDL_FreeSurface(menu);
	 SDL_FreeSurface(cursormenu);
	 SDL_FreeSurface(fontwarning);
	 SDL_FreeSurface(editsurface);
	 SDL_FreeSurface(clavier);
	 TTF_CloseFont(normal);
	 TTF_Quit();
	 SDL_Quit();
	 return 0;
}

#endif

#ifdef WII

int main(int argc, char *argv[]) {
      
     SDL_Surface *ecran=NULL;
     if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 ) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError() );
        exit(EXIT_FAILURE);
     }
     
	 fatInitDefault();	//for read files (level, serialized structs...)
	 WPAD_Init();
     
     atexit(SDL_Quit);
     SDL_ShowCursor(SDL_DISABLE);
	 
     ecran = SDL_SetVideoMode(640, 480, 32, SDL_FULLSCREEN | SDL_HWSURFACE | SDL_DOUBLEBUF);
     if (!ecran) {
        fprintf(stderr, "Unable to set video: %s\n", SDL_GetError());
        exit(EXIT_FAILURE);
     }
	 
	 TTF_Init();
	 
	 TTF_Font*normal;
	 SDL_Surface*font = NULL;
	 SDL_Surface*fontwarning = NULL;
	 SDL_Surface*editsurface = NULL;
	 SDL_Surface*cursor = NULL;	 
	 SDL_Surface*menu = NULL;
	 SDL_Surface*cursormenu = NULL;
	 SDL_Surface*clavier = NULL;
	 SDL_Surface*writing = NULL;
	 
	 normal = TTF_OpenFont("/apps/editeurlabKillHomer3D/data/calibri.ttf", 18);
	 menu = IMG_Load("/apps/editeurlabKillHomer3D/data/menu.png");
	 cursormenu = IMG_Load("/apps/editeurlabKillHomer3D/data/cursormenu.png");
	 editsurface = IMG_Load("/apps/editeurlabKillHomer3D/data/editsurface.png");
	 cursor = IMG_Load("/apps/editeurlabKillHomer3D/data/curseur.png");
	 clavier = IMG_Load("/apps/editeurlabKillHomer3D/data/keyboard.png");
	 SDL_Rect editsurfacepos = {0, 0, 70, 14};
	 SDL_Rect editcubepos = {0, 0, 14, 14};
	 SDL_Rect fontpos = {300, 20};
	 SDL_Rect fontwarningpos = {0, 150};
	 SDL_Rect menupos = {20, 20};
	 SDL_Rect cursormenupos = {30, 34};
	 SDL_Color color = {255,50,50};
	 SDL_Color warningcolor = {255, 0, 0};
	 SDL_Rect curseurnamepos = {50, 97};
	 SDL_Rect writingpos = {47, 195};
	 
	 char infoelement[64];
	 char filesaving[64];
	 int checkwarning = 0;
	 int checkplayer = 0;//variable vrifiant si tous les lmenty ont t mis sur la map avant enregistrement
	 int checkmonster = 0;
	 int checkexit = 0;
	 int i;
	 int j;
	 int elementtype = 0;//statut mur
	 int statewidth = 0;
	 int stateheight = 0;
	 int menustate = 0;
	 int menuselection = 0;
	 int delay = 0;
	 
	 int keyrepeat = 0;
	 
	 for(i=0; i<nbmaxheight; i++) {
	     for(j=0; j<nbmaxwidth; j++) {
			 editcase[i][j].x = j*14;
		     editcase[i][j].y = i*14;
			 editcase[i][j].xsprite = 14;
			 editcase[i][j].ysprite = 0;
			 editcase[i][j].wsprite = 70;
			 editcase[i][j].hsprite = 14;
			 editcase[i][j].infotype = 1;
		     editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		 }
	 }
	 
	 FILE*save = NULL;
	 
	 writing = TTF_RenderText_Solid(normal, "Write, using this keyboard, the name of your map:", color);
	 fontwarning = TTF_RenderText_Solid(normal, "Saving failed: please, verify if you have all elements' type in your map", warningcolor);
	 
	 mkdir("./data/custommap");
	 
	 while(1) {
	     WPAD_ScanPads();		 
		 
		         if(WPAD_ButtonsDown(0) & WPAD_BUTTON_A) {//cross
					     if(menustate == 0) {
							 editcase[stateheight][statewidth].statut = 1;
						     editcase[stateheight][statewidth].x = editsurfacepos.x;
						     editcase[stateheight][statewidth].y = editsurfacepos.y;
						     editcase[stateheight][statewidth].xsprite = editcubepos.x;
						     editcase[stateheight][statewidth].ysprite = editcubepos.y;
						     editcase[stateheight][statewidth].wsprite = editcubepos.w;
						     editcase[stateheight][statewidth].hsprite = editcubepos.h;
							 editcase[stateheight][statewidth].infotype = elementtype;
						 }
						 else if(menustate == 1) {
						     if(menuselection == 0) {
		                         for(i=0; i<nbmaxheight; i++) {
	                                 for(j=0; j<nbmaxwidth; j++) {
										 editcase[i][j].x = j*13;
		                                 editcase[i][j].y = i*13;
			                             editcase[i][j].xsprite = 13;
			                             editcase[i][j].ysprite = 0;
			                             editcase[i][j].wsprite = 52;
			                             editcase[i][j].hsprite = 13;
			                             editcase[i][j].infotype = 1;
		                                 editcase[i][j].statut = 0;//on autorise pas l'affiche de tous les cubes blancs pour allger le processeur :)
		                             }
	                             }
								 for(i=0; i < currentlettername-1; i++) {
								     filename[i] = '\0';
				
                				 }
								 currentletter = 0;
								 currentlettername = 0;
								 curseurnamepos.x = 50;
								 curseurnamepos.y = 97;
								 editsurfacepos.x = 0;
								 editsurfacepos.y = 0;
								 stateheight = 0;
								 statewidth = 0;
								 checkplayer = 0;
								 checkmonster = 0;
								 checkexit = 0;
	                             menustate = 0;
                             }	 
							 else if(menuselection == 1) {//selection sauvegarde
							     sprintf(filename, "name%s", filename);
								 menustate = 3;
						     }
							 else if(menuselection == 2) {
							     exit(0);
							 }
						 }
						 else if(menustate == 3) {
							 if(curseurnamepos.y == 97) { 
							     if(curseurnamepos.x > 50) {
								     if(currentlettername < 30) {
									     filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 50) {
								     if(currentlettername > 0) {   
   										 filename[currentlettername-1] = '\0';
									     currentlettername--;
									 }
								 }
							 }
							 else if(curseurnamepos.y > 97 && curseurnamepos.y < 177) {
							     if(currentlettername < 30) {
								     filename[currentlettername] = alphabet[currentletter];
								     currentlettername++;
								 }
							 }
							 else if(curseurnamepos.y == 177) {
							     if(curseurnamepos.x < 290) {
								     if(currentlettername < 30) {    
										 filename[currentlettername] = alphabet[currentletter];
									     currentlettername++;
									 }
								 }
								 else if(curseurnamepos.x == 290) {
						             if(checkplayer > 0 && checkmonster > 0 && checkexit > 0) {								 
									 sprintf(filesaving, "/apps/editeurlabKillHomer3D/data/custommap/%s.map", filename);
									 save = fopen(filesaving, "w+");
								     if(save > 0) {    
										 for(i=0; i<nbmaxheight; i++) {
						                     for(j=0; j<nbmaxwidth; j++) {
												 if(j == nbmaxwidth-1) {
										             fprintf(save, "%c\n", editcase[i][j].type);
									             }
									             if(j != nbmaxwidth-1) {
								                     fprintf(save, "%c", editcase[i][j].type);
									             }
						                     }
						                 }
									 }
						             fclose(save);
								     menustate = 0;
							         }
								     else if(checkplayer <= 0 || checkexit <= 0 || checkmonster <= 0) {
									     delay = 0;
									     checkwarning = 1;
								         menustate = 0;
								     }	
								 }
							 }
						 }
	             }
		         else if(WPAD_ButtonsDown(0) & WPAD_BUTTON_1) {//start
					     if(menustate == 0) menustate = 1;
						 else if(menustate == 1) {
						     checkplayer = 0;
							 checkmonster = 0;
							 checkexit = 0;
							 menustate = 0;
						 }
						 if(menustate == 1) {//avant d'entrer dans le menu on check si tous les paramtres (perso, monstre et sortir sont saisie) :D
								 for(i=0; i<nbmaxheight; i++) {
						             for(j=0; j<nbmaxwidth; j++) {
										 if(editcase[i][j].type == 'P') checkplayer++;
				                         if(editcase[i][j].type == '+') checkmonster++;
										 if(editcase[i][j].type == 'S') checkexit++;
						             }
						         }
						 }
                 }
		         else if(WPAD_ButtonsDown(0) & WPAD_BUTTON_B) {//L
					     if(menustate == 0) {
						     if(editcubepos.x < 56) {
						         editcubepos.x+=14;
							     elementtype++;
						     }
						     else if(editcubepos.x >= 56) {
						         editcubepos.x = 0;
							     elementtype = 0;
						     }
						 }
		         }				 
		 
		 if(WPAD_ButtonsDown(0) & WPAD_BUTTON_RIGHT && keyrepeat == 0) {//right
					     if(menustate == 0) {
						     if(editsurfacepos.x < width-14) {
						         editsurfacepos.x+=14;
							     statewidth++;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x < 290) {
						         curseurnamepos.x+=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter++;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 66) {
									     currentletter++;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 290) {
									     currentletter++;
									 }
								 }
						     }
						 }
						 keyrepeat = 5;
         }		 
		 else if(WPAD_ButtonsDown(0) & WPAD_BUTTON_LEFT && keyrepeat == 0) {
					     if(menustate == 0) {
						     if(editsurfacepos.x > 0) {
						         editsurfacepos.x-=14;
							     statewidth--;
						     }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.x > 50) {
						         curseurnamepos.x-=16;
							     if(curseurnamepos.y != 97 && curseurnamepos.y != 177) {
								     currentletter--;
								 }
								 else if(curseurnamepos.y == 97) {
								     if(curseurnamepos.x > 50) {
									     currentletter--;
									 }
								 }
								 else if(curseurnamepos.y == 177) {
								     if(curseurnamepos.x < 274) {
									     currentletter--;
									 }
								 }
						     }
						 }
						 keyrepeat = 5;
		 }
		 else if(WPAD_ButtonsDown(0) & WPAD_BUTTON_UP && keyrepeat == 0) {
					     if(menustate == 0) {
						     if(editsurfacepos.y > 0) {
						         editsurfacepos.y-=14;
							     stateheight--;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y > 34) {
							     cursormenupos.y-=14;
								 menuselection--;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y > 97) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter-=16;
						         }
							     else if(curseurnamepos.x == 50) {
							         if(curseurnamepos.y == 113) {
								         currentletter-=15;
								     }
								     else currentletter-=16;
							     }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 177) {
									     currentletter-=15;
									 }
									 else currentletter-=16;
								 }
								 curseurnamepos.y-=16;
							 }
						 }
						 keyrepeat = 5;
		 }
		 else if(WPAD_ButtonsDown(0) & WPAD_BUTTON_DOWN && keyrepeat == 0) {
					     if(menustate == 0) {
						     if(editsurfacepos.y < height-14) {
						         editsurfacepos.y+=14;
							     stateheight++;
						     }
						 }
						 else if(menustate == 1) {
						     if(cursormenupos.y < 62) {
							     cursormenupos.y+=14;
								 menuselection++;
							 }
						 }
						 else if(menustate == 3) {
						     if(curseurnamepos.y < 176) {
								 if(curseurnamepos.x != 50 && curseurnamepos.x != 290) {
								     currentletter+=16;
								 }
								 else if(curseurnamepos.x == 50) {
								     if(curseurnamepos.y == 97) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
								 else if(curseurnamepos.x == 290) {
								     if(curseurnamepos.y == 161) {
									     currentletter+=15;
									 }
									 else currentletter+=16;
								 }
							     curseurnamepos.y+=16;
						     }
						 }
						 keyrepeat = 5;
		 }	
		 else keyrepeat--;
		 if(keyrepeat < 0) keyrepeat = 0;
		 
		 SDL_FillRect(ecran, NULL, SDL_MapRGB(ecran->format, 255, 255, 255));
		 
		 if(elementtype == 0) sprintf(infoelement, "Current selection: Wall");
		 else if(elementtype == 1) sprintf(infoelement, "Current selection: Empty");
		 else if(elementtype == 2) sprintf(infoelement, "Current selection: Player");
		 else if(elementtype == 3) sprintf(infoelement, "Current selection: Homer");
		 else if(elementtype == 4) sprintf(infoelement, "Current selection: Exit");
		 font = TTF_RenderText_Solid(normal, infoelement, color);
		 
		 for(i=0; i<nbmaxheight; i++) {
	         for(j=0; j<nbmaxwidth; j++) {
				 blitCase(ecran, &editcase[i][j], editsurface);
		     }
	     }
		 
		 
		 if(menustate == 0) {
		     if(delay >= 100) {
			     checkwarning = 0;
			 }
		 }
		 else if(menustate == 3)  {
		     SDL_BlitSurface(writing, NULL, ecran, &writingpos);
			 drawKeyboard(ecran, clavier, normal, curseurnamepos);
		 }
		 
		 SDL_BlitSurface(cursor, NULL, ecran, &editsurfacepos);
		 SDL_BlitSurface(font, NULL, ecran, &fontpos);
		 SDL_FreeSurface(font);
		 if(menustate == 1) {
		     SDL_BlitSurface(menu, NULL, ecran, &menupos);
		     SDL_BlitSurface(cursormenu, NULL, ecran, &cursormenupos);
		 }
		 
		 delay++;
		 if(delay < 100 && checkwarning == 1) SDL_BlitSurface(fontwarning, NULL, ecran, &fontwarningpos);
		 
		 SDL_Flip(ecran);
	 }
	 SDL_FreeSurface(ecran);	 
	 SDL_FreeSurface(cursor);
	 SDL_FreeSurface(font);
	 SDL_FreeSurface(menu);
	 SDL_FreeSurface(cursormenu);
	 SDL_FreeSurface(fontwarning);
	 SDL_FreeSurface(editsurface);
	 SDL_FreeSurface(clavier);
	 TTF_CloseFont(normal);
	 TTF_Quit();
	 SDL_Quit();
	 return 0;
} 

#endif
