#include <pspiofilemgr.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include <math.h>
#include <pspaudio.h>
#include "mp3player.h"



#define FALSE 0
#define TRUE !FALSE
#define min(a,b) (((a)<(b))?(a):(b))
#define max(a,b) (((a)>(b))?(a):(b))
#define MadErrorString(x) mad_stream_errorstr(x)
#define INPUT_BUFFER_SIZE	(5*8192)
#define OUTPUT_BUFFER_SIZE	2048	/* Must be an integer multiple of 4. */


/* This table represents the subband-domain filter characteristics. It
* is initialized by the ParseArgs() function and is used as
* coefficients against each subband samples when DoFilter is non-nul.
*/
mad_fixed_t Filter[32];

/* DoFilter is non-nul when the Filter table defines a filter bank to
* be applied to the decoded audio subbands.
*/
int DoFilter = 0;

#define NUMCHANNELS 2
u8 *ptr;
long size;
long samplesInOutput = 0;

//////////////////////////////////////////////////////////////////////
// Global local variables
//////////////////////////////////////////////////////////////////////

//libmad lowlevel stuff

// The following variables contain the music data, ie they don't change value until you load a new file
struct mad_stream Stream;
struct mad_frame Frame;
struct mad_synth Synth;
mad_timer_t Timer;
signed short OutputBuffer[OUTPUT_BUFFER_SIZE];
unsigned char InputBuffer[INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD],
    *OutputPtr = (unsigned char *) OutputBuffer, *GuardPtr = NULL;
const unsigned char *OutputBufferEnd = (unsigned char *) OutputBuffer + OUTPUT_BUFFER_SIZE * 2;
int Status = 0, i;
unsigned long FrameCount = 0;

// The following variables are maintained and updated by the tracker during playback
static int isPlaying;		// Set to true when a mod is being played

static int myChannel;
static int eos;

#define printf	pspDebugScreenPrintf

static signed short MadFixedToSshort(mad_fixed_t Fixed)
{
    if (Fixed >= MAD_F_ONE)
	return (SHRT_MAX);
    if (Fixed <= -MAD_F_ONE)
	return (-SHRT_MAX);

    Fixed = Fixed >> (MAD_F_FRACBITS - 15);
    return ((signed short) Fixed);
}

static void MP3Callback(void *_buf2, unsigned int numSamples, void *pdata)
{
  short *_buf = (short *)_buf2;
    unsigned long samplesOut = 0;

    if (isPlaying == TRUE) {	
	if (samplesInOutput > 0) {
	    if (samplesInOutput > numSamples) {
		memcpy((char *) _buf, (char *) OutputBuffer, numSamples * 2 * 2);
		samplesOut = numSamples;
		samplesInOutput -= numSamples;
	    } else {
		memcpy((char *) _buf, (char *) OutputBuffer, samplesInOutput * 2 * 2);
		samplesOut = samplesInOutput;
		samplesInOutput = 0;
	    }
	}
	while (samplesOut < numSamples) {
	    if (Stream.buffer == NULL || Stream.error == MAD_ERROR_BUFLEN) {
		mad_stream_buffer(&Stream, ptr, size);
		Stream.error = 0;
	    }
	    if (mad_frame_decode(&Frame, &Stream)) {
		if (MAD_RECOVERABLE(Stream.error)) {
		    if (Stream.error != MAD_ERROR_LOSTSYNC || Stream.this_frame != GuardPtr) {
			sceDisplayWaitVblankStart();
		    }
		    return;
		} else if (Stream.error == MAD_ERROR_BUFLEN) {
		    eos = 1;
		    return;	
		} else {
		    sceDisplayWaitVblankStart();
		    Status = 1;
		    MP3_Stop();	//break;
		}
	    }
	    FrameCount++;
	    mad_timer_add(&Timer, Frame.header.duration);
	    mad_synth_frame(&Synth, &Frame);
	    for (i = 0; i < Synth.pcm.length; i++) {
		signed short Sample;
		if (samplesOut < numSamples) {
		    Sample = MadFixedToSshort(Synth.pcm.samples[0][i]);
		    _buf[samplesOut * 2] = Sample;
		    if (MAD_NCHANNELS(&Frame.header) == 2)
			Sample = MadFixedToSshort(Synth.pcm.samples[1][i]);
		    _buf[samplesOut * 2 + 1] = Sample;
		    samplesOut++;
		} else {
		    Sample = MadFixedToSshort(Synth.pcm.samples[0][i]);
		    OutputBuffer[samplesInOutput * 2] = Sample;
		    if (MAD_NCHANNELS(&Frame.header) == 2)
			Sample = MadFixedToSshort(Synth.pcm.samples[1][i]);
		    OutputBuffer[samplesInOutput * 2 + 1] = Sample;
		    samplesInOutput++;
		}
	    }
	}
    } else {			
	{
	    int count;
	    for (count = 0; count < numSamples * 2; count++)
		*(_buf + count) = 0;
	}
    }
}

void MP3_Init(int channel)
{
    myChannel = channel;
    isPlaying = FALSE;
    pspAudioSetChannelCallback(myChannel, MP3Callback,0);
    mad_stream_init(&Stream);
    mad_frame_init(&Frame);
    mad_synth_init(&Synth);
    mad_timer_reset(&Timer);
}


void MP3_FreeTune()
{
    if (ptr)
	free(ptr);
    mad_synth_finish(&Synth);
    mad_frame_finish(&Frame);
    mad_stream_finish(&Stream);
    if (!Status) {
	char Buffer[80];
	mad_timer_string(Timer, Buffer, "%lu:%02lu.%03u", MAD_UNITS_MINUTES, MAD_UNITS_MILLISECONDS, 0);
	//printf("%lu frames decoded (%s).\n", FrameCount, Buffer);
	sceDisplayWaitVblankStart();
	sceKernelDelayThread(500000);
    }
}


void MP3_End()
{
    MP3_Stop();
    pspAudioSetChannelCallback(myChannel, 0,0);
    MP3_FreeTune();
}

int MP3_Load(char *filename)
{
    int fd;
    eos = 0;
    if ((fd = sceIoOpen(filename, PSP_O_RDONLY, 0777)) > 0) {
	size = sceIoLseek(fd, 0, PSP_SEEK_END);
	sceIoLseek(fd, 0, PSP_SEEK_SET);
	ptr = (unsigned char *) malloc(size + 8);
	memset(ptr, 0, size + 8);
	if (ptr != 0) {		
	    sceIoRead(fd, ptr, size);
	} else {
	    printf("Error allocing\n");
	    sceIoClose(fd);
	    return 0;
	}
	sceIoClose(fd);
    } else {
	return 0;
    }
	//valeur modifie normalement sur FALSE^^ mais utile si on veut pas utiliser MP3_Play^^
    isPlaying = FALSE;
    return 1;
}

int MP3_Play()
{
    if (isPlaying)
	return TRUE;

    isPlaying = TRUE;
    return TRUE;
}

void MP3_Pause()
{
    isPlaying = !isPlaying;
}

int MP3_Stop()
{
    //stop playing
    isPlaying = FALSE;

    //clear buffer
    memset(OutputBuffer, 0, OUTPUT_BUFFER_SIZE);
    OutputPtr = (unsigned char *) OutputBuffer;
	
	//rinitialisation des mp3 pour faire la fonction relle de stop^^
	mad_stream_init(&Stream);
    mad_frame_init(&Frame);
    mad_synth_init(&Synth);
    mad_timer_reset(&Timer);

    return TRUE;
}

int MP3_EndOfStream()
{
    if (eos == 1)
	return 1;
    return 0;
}
